@@ -2,11 +2,11 @@
 Delegating package installation to pip, packaging and friends.
 """
 
-
-
 import codecs
+import collections
 import contextlib
 import importlib.util
+import inspect
 import linecache
 import os
 import platform
@@ -23,16 +23,15 @@ from functools import reduce
 from importlib import metadata
 from importlib.machinery import ModuleSpec, SourceFileLoader
 from importlib.metadata import Distribution, PackageNotFoundError, PackagePath
-from itertools import chain, product
+from itertools import chain, product, zip_longest
 from logging import getLogger
-from os.path import abspath, split
+from numbers import Complex, Integral, Number, Rational, Real
 from pathlib import Path, PureWindowsPath, WindowsPath
 from shutil import rmtree
 from sqlite3 import Cursor
 from subprocess import CalledProcessError, run
 from types import ModuleType
-from typing import (Any, Iterable, Optional, Protocol, TypeVar, Union,
-                    runtime_checkable)
+from typing import Any, Iterable, Optional, Protocol, TypeVar, Union, get_args, get_origin, runtime_checkable
 from warnings import catch_warnings, filterwarnings, warn
 
 import furl
@@ -44,11 +43,9 @@ from icontract import ensure, require
 from packaging import tags
 from packaging.specifiers import SpecifierSet
 
-from use import (Hash, InstallationError, Modes, UnexpectedHash,
-                 VersionWarning, config, sessionID)
+from use import Hash, InstallationError, Modes, UnexpectedHash, VersionWarning, config, sessionID
 from use.hash_alphabet import JACK_as_num, hexdigest_as_JACK, num_as_hexdigest
-from use.messages import (UserMessage, _web_pebkac_no_hash,
-                          _web_pebkac_no_version_no_hash)
+from use.messages import UserMessage, _web_pebkac_no_hash, _web_pebkac_no_version_no_hash
 from use.pydantics import PyPI_Project, PyPI_Release, RegistryEntry, Version
 from use.tools import pipes
 
@@ -127,12 +124,10 @@ def get_supported() -> frozenset[PlatformTag]:  # cov: exclude
                 from pip._internal.models.target_python import get_supported
         if not get_supported:
             with contextlib.suppress(ImportError):
-                from pip._internal.utils.compatibility_tags import \
-                    get_supported
+                from pip._internal.utils.compatibility_tags import get_supported
         if not get_supported:
             with contextlib.suppress(ImportError):
-                from pip._internal.resolution.resolvelib.factory import \
-                    get_supported
+                from pip._internal.resolution.resolvelib.factory import get_supported
     get_supported = get_supported or (lambda: [])
 
     items: list[PlatformTag] = [PlatformTag(platform=tag.platform) for tag in get_supported()]
@@ -273,7 +268,7 @@ def _pebkac_no_hash(
     releases = _get_releases(project)
 
     if version not in (r.version for r in releases):
-        last_version = next(iter(reversed(project.releases)))
+        last_version = project.releases >> reversed >> iter >> next
         return RuntimeWarning(Message.no_distribution_found(package_name, version, last_version))
 
     project = _filter_by_version(releases, version=version)
@@ -787,13 +782,16 @@ def _get_data_from_pypi(*, package_name: str, version: str = None) -> PyPI_Proje
 
 @beartype
 def _filter_by_platform(releases: list[PyPI_Release], *, tags: frozenset[PlatformTag]) -> list[PyPI_Release]:
-    filtered = [
-        release for release in releases if _is_compatible(release, platform_tags=tags, include_sdist=False)
-    ]
+    def compatible(info: PyPI_Release, include_sdist=False) -> bool:
+        return (
+            _is_platform_compatible(info, tags, include_sdist)
+            and not info.yanked
+            and (include_sdist or info.justuse.ext not in ("tar", "tar.gz" "zip"))
+        )
+
+    filtered = [release for release in releases if compatible(release, include_sdist=False)]
     if not filtered:
-        return [
-            release for release in releases if _is_compatible(release, platform_tags=tags, include_sdist=True)
-        ]
+        return [release for release in releases if compatible(release, include_sdist=True)]
     return filtered
 
 
@@ -858,7 +856,6 @@ def _is_platform_compatible(
         f"py{tags.interpreter_version()}",
     }
 
-
     if info.platform_tag:
         given_platform_tags = info.platform_tag.split(".") << map(PlatformTag) >> frozenset
     else:
@@ -878,15 +875,6 @@ def _is_platform_compatible(
     )
 
 
-@beartype
-def _is_compatible(info: PyPI_Release, platform_tags, include_sdist=False) -> bool:
-    return (
-        _is_platform_compatible(info, platform_tags, include_sdist)
-        and not info.yanked
-        and (include_sdist or info.justuse.ext not in ("tar", "tar.gz" "zip"))
-    )
-
-
 @beartype
 def _get_version(name: Optional[str] = None, package_name=None, /, mod=None) -> Optional[Version]:
     version: Optional[Union[Callable[...], Version, Version, str]] = None
@@ -973,3 +961,213 @@ def _fail_or_default(exception: BaseException, default: Any):
 #         proj = _filter_by_version(proj, version)
 
 #     _web_pebkac_no_hash(package_name=package_name, version=version, name=name, project=proj)
+
+
+def _real_path(*, path: Path, _applied_decorators: dict, landmark) -> tuple[str, str, str, Path]:
+    source_dir = Path.cwd()
+    # calling from another use()d module
+    # let's see where we started
+    main_mod = __import__("__main__")
+    # there are a number of ways to call use() from a non-use() starting point
+    # let's first check if we are running in jupyter
+    jupyter = "ipykernel" in sys.modules
+    # we're in jupyter, we use the CWD as set in the notebook
+    if not jupyter and hasattr(main_mod, "__file__"):
+        # problem: user wants to use.Path("some_file_in_the_same_dir")
+        # so we have to figure out where the file of the calling function is.
+        # but the *calling* function could also be a decorator, living completely elsewhere
+        # so we have to figure out whether we're being called by a decorator first.
+        # We use use.__call__ as landmark, because that's the official entry point
+        # and we can count on it being called. From there we need to check whether it was aspectized.
+        # If it was, we need to skip those decorators before finally get to the user code and
+        # we can actually see from where we've been called.
+        frame = inspect.currentframe()
+        while True:
+            if frame.f_code == landmark:
+                break
+            else:
+                frame = frame.f_back
+        # a few more steps..
+        for x in _applied_decorators:
+            frame = frame.f_back
+        try:
+            # frame is in __call__ (or the last decorator we control), we need to step one more frame back
+            source_dir = Path(frame.f_back.f_code.co_filename).resolve().parent
+        # we are being called from a shell like thonny, so we have to assume cwd
+        except OSError:
+            source_dir = Path.cwd()
+
+    if source_dir is None:
+        if main_mod.__loader__ and hasattr(main_mod.__loader__, "path"):
+            source_dir = _ensure_path(main_mod.__loader__.path).parent
+        else:
+            source_dir = Path.cwd()
+    if not source_dir.joinpath(path).exists():
+        if files := [*[*source_dir.rglob(f"**/{path}")]]:
+            source_dir = _ensure_path(files[0]).parent
+        else:
+            source_dir = Path.cwd()
+    if not source_dir.exists():
+        raise NotImplementedError("Can't determine a relative path from a virtual file.")
+    if not path.exists():
+        path = source_dir.joinpath(path).resolve()
+    if not path.exists():
+        raise ImportError(f"Sure '{path}' exists?")
+    if not path.is_absolute():
+        path = path.resolve()
+    try:
+        name = path.relative_to(source_dir)
+    except ValueError:
+        source_dir = path.parent
+        os.chdir(source_dir)
+        name = path.relative_to(source_dir)
+    ext = name.as_posix().rpartition(".")[-1]
+    name_as_path_with_ext = name.as_posix()
+    name_as_path = name_as_path_with_ext[: -len(ext) - (1 if ext else 0)]
+    name = name_as_path.replace("/", ".")
+    name_parts = name.split(".")
+    package_name = ".".join(name_parts[:-1])
+    module_name = path.stem  # sic!
+
+    return name, module_name, package_name, path
+
+
+numerics = [bool, Integral, Rational, Real, Complex]
+
+
+class NotComparableWarning(UserWarning):
+    pass
+
+
+def _modules_are_compatible(pre, post):
+    for name, pre_obj in pre.__dict__.items():
+        if callable(pre_obj):
+            try:
+                post_obj = getattr(post, name)
+            except AttributeError:
+                return False
+            if _is_compatible(pre_obj, post_obj):
+                continue
+            else:
+                return False
+    return True
+
+
+def _is_compatible(pre, post):
+    sig = inspect.signature(pre)
+    pre_sig = []
+    # first separate args and kwargs so we can sort kwargs alphabetically
+    args = []
+    kwargs = []
+
+    for k, v in sig.parameters.items():
+        if v.kind is v.VAR_KEYWORD or v.POSITIONAL_OR_KEYWORD:
+            kwargs.append((k, v))
+        else:
+            args.append((k, v))
+
+    for k, v in args:
+        v = v.annotation
+        pre_sig.append(v if v is not inspect._empty else Any)
+
+    for k, v in sorted(kwargs):
+        v = v.annotation
+        pre_sig.append(v if v is not inspect._empty else Any)
+
+    v = sig.return_annotation
+    pre_sig.append(v if v is not inspect._empty else Any)
+
+    sig = inspect.signature(post)
+    post_sig = []
+
+    args = []
+    kwargs = []
+
+    for k, v in sig.parameters.items():
+        if v.kind is v.VAR_KEYWORD or v.POSITIONAL_OR_KEYWORD:
+            kwargs.append((k, v))
+        else:
+            args.append((k, v))
+
+    for k, v in args:
+        v = v.annotation
+        post_sig.append(v if v is not inspect._empty else Any)
+
+    for k, v in sorted(kwargs):
+        v = v.annotation
+        post_sig.append(v if v is not inspect._empty else Any)
+
+    v = sig.return_annotation
+    post_sig.append(v if v is not inspect._empty else Any)
+    return all(_check(x, y) for x, y in zip_longest(pre_sig, post_sig, fillvalue=Any))
+
+
+def _check(x, y):
+    if config.debugging:
+        print(
+            f"checking:\n  "
+            f"{x!r:<30} "
+            f"({type(x).__module__}.{type(x).__qualname__})\n  "
+            f"{y!r:<30} "
+            f"({type(y).__module__}.{type(y).__qualname__})"
+        )
+
+    # narrowing {(Any => Any) | (Any => something)} is OK
+    if x is Any:
+        return True
+    # broadening (something => Any) is NOK
+    if y is Any:
+        return False
+
+    # now to the more specific cases (something => something else)
+    with contextlib.suppress(TypeError):  # issubclass is allergic to container classes (types.GenericAlias)
+        # let's first check if we're dealing with numbers
+        if issubclass(x, Number) and issubclass(y, Number):
+            # since the generic implementations aren't actual subclasses of each other
+            # we have to map to the numeric tower classes
+            for X in numerics:
+                if issubclass(x, X):
+                    break
+            x = X
+            for Y in numerics:
+                if issubclass(y, Y):
+                    break
+            y = Y
+    # Need to do this *before* we access `__name__`
+    X = get_origin(x) or x
+    Y = get_origin(y) or y
+
+    # the other important type hierarchy are
+    # containers
+    # let's check if the user is using typing
+    # classes and educate them to use
+    # collections.abc instead
+    for name, t in {"X": X, "Y": Y}.items():
+        if t.__module__ != "typing":
+            continue
+        tca = getattr(collections.abc, t.__name__, 0)
+        if not tca:
+            continue
+        warn(
+            NotComparableWarning(
+                f"{t} is of a type from the typing "
+                " module, which can't be compared. "
+                "Please use a type from the "
+                "`collections.abc` module (`{tca!r}`)."
+            )
+        )
+        locals().__setitem__(name, tca)
+
+    if X is not None and Y is not None:
+        # (Sequence => list) is OK, (list => Sequence) is NOK
+        return (
+            all(_check(x_, y_) for x_, y_ in zip_longest(get_args(x), get_args(y), fillvalue=Any))
+            if issubclass(Y, X)
+            else False
+        )
+
+    try:
+        # (x => y) where y is not a subclass of x is NOK
+        return issubclass(y, x)
+    except TypeError:  # (int => list[int]) or (list[int] => int) NOK
+        return False
